/* lenstring.h --- strings with explicit lengths
   Jim Blandy <jimb@gnu.ai.mit.edu> --- July 1994 */

#ifndef LENSTRING_H
#define LENSTRING_H

#include <stdio.h>

/* Here's our representation of a string.  It's a pain, but we don't
   want to be confused by null characters.  To make it easier to treat
   lenstrings as C strings (when possible), text[len] should always be
   '\0'.  */
struct lenstring
{
  char *text;
  size_t len;
};

/* Copy a block of memory, and initialize a lenstring structure to
   point to it.  */
extern void make_lenstring (struct lenstring *LENSTRING,
                            char *TEXT, size_t LEN);


/* Copy the LEN bytes at TEXT to the end of LENSTRING; realloc
   LENSTRING if necessary.  */
extern void append_to_lenstring (struct lenstring *LENSTRING,
				 char *TEXT, size_t LEN);


/* Read strings (delimited as described in protocol.texi) from
   SOURCE_FILE, until we find an unescaped newline.

   If we hit EOF before reading anything interesting, return EOF;
   don't modify *STRINGS_OUT or *TEXT_OUT.

   If we read some strings, set *STRINGS_OUT to a malloc'ed array of
   lenstrings containing the strings, and return the number of strings
   we read.  Note that the text for all the lenstrings is actually
   kept in one big malloc'ed block; *TEXT_OUT is set to the address of
   this block.  The caller is responsible for freeing *STRINGS_OUT and
   *TEXT_OUT.  */
extern int getstrings (FILE *SOURCE_FILE,
		       struct lenstring **STRINGS_OUT,
		       char **TEXT_OUT);


/* Write the LEN bytes at TEXT to OUTPUT_FILE, using the quoting
   syntax described in protocol.texi.  This doesn't really belong
   here, as lenstrings aren't involved, but if you think about this
   module as dealing with fully-transparent strings, it's okay.  */
extern void write_quoted (FILE *OUTPUT_FILE,
			  const char *TEXT,
			  size_t LEN);

/* Like write_quoted, but for lenstrings.  */
extern void write_quoted_lenstring (FILE *OUTPUT_FILE,
				    struct lenstring *LENSTRING);


#endif /* LENSTRING_H */
