;; gene-help.el (part of ale).
;; Copyright (C)  1994  Karl Fogel <kfogel@darwin.life.uiuc.edu>
;;
;; This software is copyrighted under the terms of the GNU General
;; Public License (see terms below).  The copyright covers this file,
;; specifically, and the program of which it is a part.
;; 
;;      This program is free software; you can redistribute it and/or modify
;;      it under the terms of the GNU General Public License as published by
;;      the Free Software Foundation; either version 1, or (at your option)
;;      any later version.
;; 
;;      This program is distributed in the hope that it will be useful,
;;      but WITHOUT ANY WARRANTY; without even the implied warranty of
;;      MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;;      GNU General Public License for more details.
;; 
;;      You should have received a copy of the GNU General Public License
;;      along with this program; if not, write to the Free Software
;;      Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
;;
;; Search for instances of "todo:" to find notes about future plans.



;;; Code:
(require 'gene-frame)


;; I'm worried about the reliability of gene-frame.el, because it
;; depends on frames being named after their buffers.  Isn't that
;; something the user can override?  Also, it can end up creating too
;; many frames, if the frame/buffer correspondences somehow get messed
;; up.  Hmmm, better talk to Jim about this.

(defconst gene-help-frame-parameters
  `((foreground-color . ,gene-foreground-color)
    (background-color . ,gene-background-color)
    (cursor-color     . ,gene-cursor-color)
    (mouse-color      . ,gene-cursor-color)
    (unsplittable     . t)
    (menu-bar-lines   . 1))
  "Parameters for help frames.")


(defun gene-help-quick-reference ()
  "Display a quick reference in the Help frame."
  (interactive)
  (let* ((name "Quick Reference")
         (frame (gene-frame-show name gene-help-frame-parameters)))
    (select-frame frame)
    (set-buffer (get-buffer-create name))
    (erase-buffer)
    (setq mode-line-format "Quick Reference")
    (insert 

"Movement and editing commands:
-------------------------------

C-f, right-arrow ................ forward position
C-b, left-arrow ................. backward position
C-n, down-arrow ................. next line
C-p, up-arrow ................... previous line

C-right-arrow ................... forward to next base
C-left-arrow .................... backward to next base
M-right-arrow ................... forward to next base group
M-left-arrow .................... backward to previous base group

C-M-right-arrow ................. forward screen
C-M-left-arrow .................. backward screen
S-M-right-arrow ................. go to last base in sequence
S-M-left-arrow .................. go to first base in sequence

C-down-arrow .................... down to base
C-up-arrow ...................... up to base
M-down-arrow .................... down to different base
M-up-arrow ...................... up to different base

C-M-down-arrow .................. down screen
C-M-up-arrow .................... up screen
S-M-down-arrow .................. down to last sequence
S-M-up-arrow .................... up to first sequence

M-? ............................. show position info

C-o ............................. recenter

C-g ............................. panic button (quits anything)

mouse-1 ......................... go to the clicked-on spot
double-mouse-1 .................. go to that spot, and then
                                  recenter horizontally
C-r ............................. slide base(s) to the right
C-l ............................. slide base(s) to the left
M-r ............................. fetch base from right
M-l ............................. fetch base from left
C-M-r ........................... throw base(s) to the right
C-M-l ........................... throw base(s) to the left

C-d, delete ..................... delete char in front of point
Backspace ....................... delete char behind point

M-u ............................. undo last change(s)

Tab ............................. insert context-sensitive indel.
M-Tab ........................... same, but in other sequences.

mouse-2 ......................... slide sequence chunks right or left.

Selections:
-----------
S-mouse-1 ....................... define first corner or side
mouse-1 ......................... (after S-mouse-1) set other corner
S-mouse-1 ....................... (after S-mouse-1) set other side
M-mouse-1 ....................... (after S-mouse-1) define other corner,
                                  for a hard-edged selection.
")

    (goto-char (point-min))
    (gene-menu-bar-make-hide-button)
    (set-window-buffer
     (frame-selected-window frame)
     (current-buffer))))


;;; Info
(defun gene-help-info ()
  "Bring up the Ale info documentation in a frame."
  (interactive)
  (require 'info)
  (let ((frame (gene-frame-show "Online Manual" gene-help-frame-parameters)))
    (select-frame frame)
    (Info-find-node "ale.info" "top")
    (gene-menu-bar-make-hide-button)
    (set-window-buffer
     (frame-selected-window frame)
     (current-buffer))))


(defun gene-help-make-faces ()
  "Set up faces for gene help buffers.
Right now, this only exists to set a pleasanter background for the
info highlight face."
  (set-face-background 'highlight (gene-check-color-allowable "rosybrown")))



(provide 'gene-help)
